@@ -21,8 +21,9 @@
 int utf8_to_ucs2 (const unsigned char * input, const unsigned char ** end_ptr)
 {
     *end_ptr = input;
-    if (input[0] == 0)
+    if (input[0] == 0) {
         return -1;
+    }
     if (input[0] < 0x80) {
 	/* One byte (ASCII) case. */
         * end_ptr = input + 1;
@@ -57,8 +58,8 @@ int utf8_to_ucs2 (const unsigned char * input, const unsigned char ** end_ptr)
 
    Output: UTF-8 characters in buffer "utf8". 
 
-   Return value: the number of bytes written into "utf8", or -1 if
-   there was an error.
+   Return value: the number of bytes written into "utf8", or a
+   negative number if there was an error.
 
    This adds a zero byte to the end of the string. It assumes that the
    buffer "utf8" has at least four bytes of space to write to. */
